#include "mqtt.hpp"

TaskHandle_t mqttTaskHandle = NULL;

// WiFiClient wifiClient;
//PubSubClient client(wifiClient);
PsychicMqttClient mqttClient;

// avoid circular deps, just forward declare externs used here.
#ifdef HAS_FRONTLIGHT
bool hasLightLevel();
float getLightLevel();
#endif
String getMyHostname();

void onMqttCallback(char* topic, byte* payload, unsigned int length)
{
    Serial.println("MQTT message arrived");
}

const String getDeviceTopic()
{
    const String hostname = getMyHostname();
    const String rootTopic = preferences.getString("mqttRootTopic", DEFAULT_MQTT_ROOTTOPIC);
    String fullTopic = rootTopic;
    if (!rootTopic.endsWith("/") && rootTopic != "")
    {
        fullTopic += "/";
    }
    fullTopic += hostname + "/";

    return String(fullTopic);
}

// boolean connectMqtt()
// {
//     const String willTopic = getDeviceTopic() + "status";
//     mqttClient.connect()
//     boolean result = client.connect("btclockClient", willTopic.c_str(), 0, true, "offline");
//     if (!result)
//     {
//         Serial.println("[MQTT] could not connect");
//         return result;
//     }
//     publish("status", "online", true);
//     return result;
// }

boolean setupMqtt()
{
    const String url = preferences.getString("mqttUrl", DEFAULT_MQTT_URL);
    if (url == "")
    {
        Serial.println("[MQTT] url not set");
        return false;
    }
    Serial.print("[MQTT] url: ");
    Serial.println(url.c_str());

    mqttClient.setClientId(getMyHostname().c_str());
    mqttClient.setServer(url.c_str());
    // if (url.startsWith("mqtts:") || url.startsWith("wss:"))
    //     mqttClient.attachArduinoCACertBundle();
    mqttClient.connect();

    const String willTopic = getDeviceTopic() + "status";
    mqttClient.setWill(willTopic.c_str(), 0, 1, "offline");
    publish("status", "online", true);

    return true;
}

void mqttTask(void *pvParameters)
{
    int t=0;
    while (1)
    {
        // client.loop();
        vTaskDelay(pdMS_TO_TICKS(1000));
        if (t++ % 10 == 0)
        {
#ifdef HAS_FRONTLIGHT
            if (hasLightLevel())
            {
                std::string lux_s = std::to_string(static_cast<int>(std::round(getLightLevel())));
                publish("sensors/lux", lux_s.c_str());
            }
#endif
            if (WiFi.isConnected())
            {
                int8_t rssi = WiFi.RSSI();
                std::string rssi_s = std::to_string(static_cast<int>(rssi));
                publish("wifi/rssi", rssi_s.c_str());
                publish("wifi/bssid", WiFi.BSSIDstr().c_str());
            }
            std::string heap_free_s = std::to_string(static_cast<int>(ESP.getFreeHeap()));
            publish("mem/heap_free", heap_free_s.c_str());
            std::string heap_size_s = std::to_string(static_cast<int>(ESP.getHeapSize()));
            publish("mem/heap_size", heap_size_s.c_str());
        }
    }
}

void setupMqttTask()
{
    xTaskCreate(mqttTask, "mqttTask", 8192, NULL, 10, &mqttTaskHandle);
}

void publishForDevice(const char *topic, const char *payload)
{
    publishForDevice(topic, payload, false);
}

void publishForDevice(const char *topic, const char *payload, boolean retain)
{
    const String fullTopic = getDeviceTopic() + topic;
    publish(fullTopic.c_str(), payload, retain);
}

void publish(const char *topic, const char *payload)
{
    publish(topic, payload, false);
}

void publish(const char *topic, const char *payload, boolean retain)
{
    if (!mqttClient.publish(topic, 0, retain, payload))
    {
        Serial.println("[MQTT] could not write");
    }
}